package javax.swing;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.Window;
import java.util.HashSet;
import java.awt.Container;
import java.awt.Component;
import java.util.HashMap;
import java.io.Serializable;
import java.awt.FocusTraversalPolicy;

final class LegacyGlueFocusTraversalPolicy extends FocusTraversalPolicy implements Serializable
{
    private transient FocusTraversalPolicy delegatePolicy;
    private transient DefaultFocusManager delegateManager;
    private HashMap<Component, Component> forwardMap;
    private HashMap<Component, Component> backwardMap;
    
    LegacyGlueFocusTraversalPolicy(FocusTraversalPolicy delegatePolicy) {
        this.forwardMap = new HashMap();
        this.backwardMap = new HashMap();
        this.delegatePolicy = delegatePolicy;
    }
    
    LegacyGlueFocusTraversalPolicy(DefaultFocusManager delegateManager) {
        this.forwardMap = new HashMap();
        this.backwardMap = new HashMap();
        this.delegateManager = delegateManager;
    }
    
    void setNextFocusableComponent(Component component, Component component2) {
        this.forwardMap.put(component, component2);
        this.backwardMap.put(component2, component);
    }
    
    void unsetNextFocusableComponent(Component key, Component key2) {
        this.forwardMap.remove(key);
        this.backwardMap.remove(key2);
    }
    
    @Override
    public Component getComponentAfter(Container container, Component e) {
        HashSet<Component> set = new HashSet();
        do {
            e = (Component)this.forwardMap.get(e);
            if (e == null) {
                if (this.delegatePolicy != null && e.isFocusCycleRoot(container)) {
                    return this.delegatePolicy.getComponentAfter(container, e);
                }
                if (this.delegateManager != null) {
                    return this.delegateManager.getComponentAfter(container, e);
                }
                return null;
            }
            else {
                if (set.contains(e)) {
                    return null;
                }
                set.add(e);
            }
        } while (!this.accept(e));
        return e;
    }
    
    @Override
    public Component getComponentBefore(Container container, Component e) {
        HashSet<Component> set = new HashSet();
        do {
            e = (Component)this.backwardMap.get(e);
            if (e == null) {
                if (this.delegatePolicy != null && e.isFocusCycleRoot(container)) {
                    return this.delegatePolicy.getComponentBefore(container, e);
                }
                if (this.delegateManager != null) {
                    return this.delegateManager.getComponentBefore(container, e);
                }
                return null;
            }
            else {
                if (set.contains(e)) {
                    return null;
                }
                set.add(e);
            }
        } while (!this.accept(e));
        return e;
    }
    
    @Override
    public Component getFirstComponent(Container container) {
        if (this.delegatePolicy != null) {
            return this.delegatePolicy.getFirstComponent(container);
        }
        if (this.delegateManager != null) {
            return this.delegateManager.getFirstComponent(container);
        }
        return null;
    }
    
    @Override
    public Component getLastComponent(Container container) {
        if (this.delegatePolicy != null) {
            return this.delegatePolicy.getLastComponent(container);
        }
        if (this.delegateManager != null) {
            return this.delegateManager.getLastComponent(container);
        }
        return null;
    }
    
    @Override
    public Component getDefaultComponent(Container container) {
        if (this.delegatePolicy != null) {
            return this.delegatePolicy.getDefaultComponent(container);
        }
        return this.getFirstComponent(container);
    }
    
    private boolean accept(Component component) {
        if (!component.isVisible() || !component.isDisplayable() || !component.isFocusable() || !component.isEnabled()) {
            return false;
        }
        if (!(component instanceof Window)) {
            Container container = component.getParent();
            while (container != null) {
                if (!container.isEnabled() && !container.isLightweight()) {
                    return false;
                }
                if ((container instanceof Window)) {
                    break;
                }
                container = container.getParent();
            }
        }
        return true;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if ((this.delegatePolicy instanceof Serializable)) {
            objectOutputStream.writeObject(this.delegatePolicy);
        }
        else {
            objectOutputStream.writeObject(null);
        }
        if ((this.delegateManager instanceof Serializable)) {
            objectOutputStream.writeObject(this.delegateManager);
        }
        else {
            objectOutputStream.writeObject(null);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.delegatePolicy = (FocusTraversalPolicy)objectInputStream.readObject();
        this.delegateManager = (DefaultFocusManager)objectInputStream.readObject();
    }
}
